/* $Id: Arc.h,v 1.3 1992/12/11 08:34:19 jojic Exp $ */
/*****************************************************************************
         Copyright (c) BULL S.A. 1990

MODULE:  Graphic Objects
FILE:    Arc.h
DATE:    Feb 1991

DESCRIPTION:
    This is one of the basic graphic objects.
    It simply draws an ellipse arc with the following characteristics
        o  the ellipse axises are the GoArc coordinate system axises
        o  the ellipse radii are _radiusX and _radiusY,
	   with respect to the axises.
	o  the arc is drawn starting at the angle _start.
	o  the arc has an amplitude (angle) of _amplitude
	      - positive value means clockwise        (1*)
	      - negative value means counterclockwise (1*)
	o  the boolean attribute _pie tells if the arc is delimited by its
	   radii segments (TRUE) or by its chord (FALSE). This value affects
	   the arc outline only if the parameter _closed is set to TRUE, but
	   it always affects the background shape (filled arc).
	o  the boolean attribute _closed tells if the segment(s) (radii if _pie
	   or chord if not) are to be drawn when the arc has an outline (see
	   GoGeometric::outline()).

(1*) Remember that coordinate systems are indirect
        O
         *----->
         |      X
         |
         V
           Y

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _GoArc_h
#define _GoArc_h

#include <Go/Geometric.h>
#include <Go/mArc.h>

class GoArc : public GoGeometric {
    friend class GoArcMetaclass;

  public:
    
    static GoArcMetaclass* metaclass();
    static GoArc* create() { return (GoArc*) Go::Create(GoArc::metaclass()); }

    static GoArc* create(GNumber radiusX, GNumber radiusY = 1.,
			 GNumber amplitude = 2*PI, GNumber start = 0.);

    GoArc(GNumber radiusX = 1., GNumber radiusY = 1.,
	  GNumber amplitude = 2*PI, GNumber start = 0.);

    GoArc* dupAsArc() const;
    
    GNumber start()     const { return _start; }
    GNumber amplitude() const { return _amplitude; }
    GNumber radiusX()   const { return _radX; }
    GNumber radiusY()   const { return _radY; }
    GBool   pie()       const { return _pie; }
    GBool   closed()    const { return _closed; }

    virtual GBool start(GNumber angle);
    virtual GBool amplitude(GNumber angle);
    virtual GBool radiusX(GNumber radius);
    virtual GBool radiusY(GNumber radius);
    virtual GBool reset(GNumber, GNumber, GNumber, GNumber);
    virtual GBool pie(GBool);
    virtual GBool closed(GBool);

    // pre/post conditions;
    void radiusXPrecond(GBool func(void*, const GoArc*, GNumber*),
			void* closure, GAddOrRemove = GADD);
    void radiusXPostcond(void func(void*, GoArc*),
			 void* closure, GAddOrRemove = GADD);
    void radiusYPrecond(GBool func(void*, const GoArc*, GNumber*),
			void* closure, GAddOrRemove = GADD);
    void radiusYPostcond(void func(void*, GoArc*),
			 void* closure, GAddOrRemove = GADD);
    void amplitudePrecond(GBool func(void*, const GoArc*, GNumber*),
			  void* closure, GAddOrRemove = GADD);
    void amplitudePostcond(void func(void*, GoArc*),
			   void* closure, GAddOrRemove = GADD);
    void startPrecond(GBool func(void*, const GoArc*, GNumber*),
		      void* closure, GAddOrRemove = GADD);
    void startPostcond(void func(void*, GoArc*),
		       void* closure, GAddOrRemove = GADD);
    void resetPrecond(GBool func(void*, const GoArc*,
				GNumber*, GNumber*, GNumber*, GNumber*),
		      void* closure, GAddOrRemove = GADD);
    void resetPostcond(void func(void*, GoArc*),
		       void* closure, GAddOrRemove = GADD);
    
    void piePrecond(GBool func(void*, const GoArc*, GBool*),
		    void* closure, GAddOrRemove = GADD);
    void piePostcond(void func(void*, GoArc*),
		     void* closure, GAddOrRemove = GADD);
    void closedPrecond(GBool func(void*, const GoArc*, GBool*),
		       void* closure, GAddOrRemove = GADD);
    void closedPostcond(void func(void*, GoArc*),
			void* closure, GAddOrRemove = GADD);

    // call preconditions in public;
    GBool callRadiusXPrecond(GNumber*) const;
    GBool callRadiusYPrecond(GNumber*) const;
    GBool callAmplitudePrecond(GNumber*) const;
    GBool callStartPrecond(GNumber*) const;
    GBool callResetPrecond(GNumber* radx, GNumber* rady,
			   GNumber* amplitude, GNumber* start) const;
    GBool callPiePrecond(GBool*) const;
    GBool callClosedPrecond(GBool*) const;

    // save/restore;
    static int	 save(const GoArc*, GoStream*);
    static int	 restore(GoArc**, GoStream*);   // creates + restores;
    static int	 readParams(GoArc*, GoStream*); // restores only;

  protected:


    virtual GBool m_contains(const GPoint&, GNumber maxDist = 0) const;
    virtual void m_add_clip(GRegion&) const;
    virtual void m_add_opaque_clip(GRegion&) const;
    virtual void m_draw(const GRegion&) const;
    virtual void m_wired_draw(const GCoordSys&, const GPen&, GQuality) const;
    virtual void m_dup(const Go* src, Go*, Go*, GBool, GBool);

    GBool Reset(GNumber, GNumber, GNumber, GNumber);
    void _reset(GNumber radX, GNumber radY, GNumber amplitude, GNumber start) {
	_radX = radX; _radY = radY; _amplitude = amplitude; _start = start;
    }

  private:
    
    GNumber _start, _amplitude;   // arc drawn between start and start+amplitude;
    GNumber _radX, _radY;         // radii;

    GBool _pie			 :1; /* arc pie fill rule if true;
				      * else arc chord;
				      */
    GBool _closed		 :1; /* if true, close arc outline according
	                              * to the value of _pie,
				      * else use outline elliptical part only;
				      */
    
    // pre/post condition flags;
    GBool _has_radiusX_precond    :1;
    GBool _has_radiusX_postcond   :1;
    GBool _has_radiusY_precond    :1;
    GBool _has_radiusY_postcond   :1;
    GBool _has_amplitude_precond  :1;
    GBool _has_amplitude_postcond :1;
    GBool _has_start_precond      :1;
    GBool _has_start_postcond     :1;
    GBool _has_reset_precond      :1;
    GBool _has_reset_postcond     :1;

    GBool _has_pie_precond	 :1;
    GBool _has_pie_postcond	 :1;
    GBool _has_closed_precond	 :1;
    GBool _has_closed_postcond	 :1;
    
    // call pre/post conditions;
    GBool _call_radiusX_precond(GNumber*) const;
    GBool _call_radiusY_precond(GNumber*) const;
    GBool _call_amplitude_precond(GNumber*) const;
    GBool _call_start_precond(GNumber*) const;
    GBool _call_reset_precond(GNumber*, GNumber*, GNumber*, GNumber*) const;
    GBool _call_pie_precond(GBool*) const;
    GBool _call_closed_precond(GBool*) const;
    void _call_radiusX_postcond();
    void _call_radiusY_postcond();
    void _call_amplitude_postcond();
    void _call_start_postcond();
    void _call_reset_postcond();
    void _call_pie_postcond();
    void _call_closed_postcond();

    // call pre/post conditions;
    GBool _call_class_radiusX_precond(GNumber*) const;
    GBool _call_class_radiusY_precond(GNumber*) const;
    GBool _call_class_amplitude_precond(GNumber*) const;
    GBool _call_class_start_precond(GNumber*) const;
    GBool _call_class_reset_precond(GNumber*,GNumber*,GNumber*,GNumber*) const;
    GBool _call_class_pie_precond(GBool*) const;
    GBool _call_class_closed_precond(GBool*) const;
    void _call_class_radiusX_postcond();
    void _call_class_radiusY_postcond();
    void _call_class_amplitude_postcond();
    void _call_class_start_postcond();
    void _call_class_reset_postcond();
    void _call_class_pie_postcond();
    void _call_class_closed_postcond();

    virtual GoMetaclass* m_metaclass() const;
    virtual void         m_check() const;

    GBool _set_pie(GBool);
    GBool _set_closed(GBool);
};

inline GBool GoArc::callRadiusXPrecond(GNumber* radx) const
{
    return _call_radiusX_precond(radx);
}

inline GBool GoArc::callRadiusYPrecond(GNumber* rady) const
{
    return _call_radiusY_precond(rady);
}

inline GBool GoArc::callAmplitudePrecond(GNumber* amplitude) const
{
    return _call_amplitude_precond(amplitude);
}

inline GBool GoArc::callStartPrecond(GNumber* start) const
{
    return _call_start_precond(start);
}

inline GBool GoArc::callResetPrecond(GNumber* radx, GNumber* rady,
			      GNumber* amplitude, GNumber* start) const
    
{
    return _call_reset_precond(radx, rady, amplitude, start);
}

inline GBool GoArc::callPiePrecond(GBool* pie) const
{
    return _call_pie_precond(pie);
}

inline GBool GoArc::callClosedPrecond(GBool* closed) const
{
    return _call_closed_precond(closed);
}


#endif /* _GoArc_h */



