/* $Id: Cubic.h,v 1.3 1992/12/11 08:34:31 jojic Exp $ */
/*****************************************************************************
MODULE:  Geometric
FILE:    Cubic.h
AUTHOR:  Gilles Benati
DATE:    September 20th, 1991

DESCRIPTION:
    
	A longer curve can be made up of series of curve segments joining at the
  end points. In the case of cubic segmented curves each segment of the curve
  is determined by an evaluation window of exactly four control points. The
  first four control points describe the first segment of the curve, then this
  evaluation window shifts to next series of four points. The appropriate shift
  in control points between segments is an important attribute of a curve type
  which will be referred to as evaluation shift from now onwards.

	The points on the curve segment are calculated using the basis matrix
  of the particular type of the curve and the geometric vector of the
  corresponding control points. Choosing the basis matrix suitably, we can have
  various types of interpolating and approximating splines. The closeness of
  the curve to the control polygon can be varied by making the basis matrix as
  function of another parameter known as 'tension'. A very tense spline looks
  more like its control polygon than an untense spline.

	GoCubic implements the common functions for all the cubic spline
  subclasses. Each subclass has to provide the fuction to initialize the basis
  matrix when requested. The subclass can introduce additional parameters for
  the basis matrix (eg, some of the splines like Beta Spline and Tau Spline
  have a parameter called bias) or ignore the tension parameter if not
  interested. A subclass whose evaluation shift is different from 1 has to
  implement the virtual function evalShift() as the default value of the
  evaluation shift for the GoCubic class is 1.


COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#ifndef _GoCubic_h
#define _GoCubic_h

#include <Go/Curve.h>
#include <Go/mCubic.h>

class GoCubic : public GoCurve {
    friend class GoCubicMetaclass;

  public:
   
    static GoCubicMetaclass* metaclass();
  
    // pre/post conditions;
    void tensionPrecond(GBool func(void*, const GoCubic*, GNumber*),
			void* closure, GAddOrRemove = GADD);
    void tensionPostcond(void func(void*, GoCubic*),
			 void* closure, GAddOrRemove = GADD);

    GNumber tension() const { return _tension; }
    virtual GBool tension(GNumber ts);
    virtual GBool delPoint(int);
    
    static int  save(const GoCubic*, GoStream*);
//  static int  restore(GoCubic**, GoStream*);
    static int  readParams(GoCubic*, GoStream*);

  protected:
    
    GNumber _coeffMatrix[4][4];
    GNumber _tension;
    GBool   _inited;

    GoCubic(int npoint = 0, const GPoint* points = NULL);

    virtual int		evalShift() const; // {return 1;};
    virtual void	initCoeffMatrix(); // {_inited = GTRUE;};
    virtual GPoint*	getCurvePoints(int&, int nb_ctrl,
				       const GPoint* ctrl) const;

//  virtual GBool m_contains(const GPoint&, GNumber maxDist = 0) const;
    virtual void m_draw(const GRegion&) const;
//  virtual void m_wired_draw(const GRegion&, const GPen&, GQuality) const;
//  virtual void m_add_clip(GRegion&) const;
//  virtual void m_add_opaque_clip(GRegion&) const;
    virtual void m_dup(const Go*, Go*, Go*, GBool, GBool);

  private:

    // pre/post condition flags;
    GBool _has_tension_precond  :1;
    GBool _has_tension_postcond :1;

    // call pre/post conditions;
    GBool _call_tension_precond(GNumber*) const;
    void _call_tension_postcond();

    // call metaclass pre/post conditions;
    GBool _call_class_tension_precond(GNumber*) const;
    void _call_class_tension_postcond();

    virtual GoMetaclass* m_metaclass() const;
    virtual void         m_check() const;

    GBool _set_tension(GNumber);
};

#endif /* _GoCubic_h */


