/* $Id: GoRoot.h,v 1.5 1992/12/11 08:34:48 jojic Exp $ */
/*****************************************************************************
MODULE:  Go Kernel
FILE:    GoRoot.h
AUTHOR:  Oliver Jojic
DATE:    July 90

DESCRIPTION:
  A GoRoot is the root of a go tree.
  (but the root of a go tree may not be a GoRoot).

  To get drawings, your go tree must have a GoRoot.
  The GoRoot itself is attached to a GoInterf (X11, Windows, ...),
  on which drawings are done.

  Nothing is drawn until someone calls "goroot->flush()".
  A possible event loop:
  	for (;;)
	{
	    if (!goroot->interf()->eventReady())
	        goroot->flush();
	    goroot->interf()->handleNextEvent();
	}

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _GoRoot_
#define _GoRoot_

#include <Go/Go.h>
#include <Go/Tools/Array.h>

class GExposeEvent;

class GoRoot : public Go
{
  public:

    static GoRoot* create ();
    static GoRoot* create (GoInterf*);

	       GoRoot ();
    virtual   ~GoRoot ();

    GoInterf*  interf () const { return _interf; }
    GBool      interf (GoInterf*);

    void       flush ();
    void       redrawForeOnly (Go*, const GRegion& = *(GRegion*)NULL);
    void       redrawBackAndFore (const GRegion&);

    void       flushPrecond (void (*precond)(void*, GoRoot*, const GRegion*),
			     void* closure, GAddOrRemove = GADD);
    void       flushPostcond (void (*postcond)(void*, GoRoot*, const GRegion*),
			      void* closure, GAddOrRemove=GADD);

    int        maxLowerOpaqueSearchCount() const { return max_opaque_search; }
    void       maxLowerOpaqueSearchCount (int n) { max_opaque_search = n; }
    int	       maxBoxesCount () const { return max_boxes_count; }
    void       maxBoxesCount (int m) { max_boxes_count = m; }
    GNumber    minOpaqueArea () const { return sqrt(min_opaque_square_area); }
    void       minOpaqueArea (GNumber a) { min_opaque_square_area = a*a; }

    void       callClipPostconditions() { setup_clips(); }

    static GoMetaclass* metaclass();
    
    void       disableDamage () { damage_enabled = GFALSE; }
    void       enableDamage () { damage_enabled = GTRUE; }
    GBool      damageEnabled () const { return damage_enabled; }

  private:

    friend Go;

    virtual GoMetaclass* m_metaclass () const;
    virtual void         m_dup (const Go*, Go*, Go*, GBool, GBool);
    virtual void	 m_draw (const GRegion&) const;
    virtual void	 m_add_clip (GRegion&) const;
    virtual GAssoc* 	 d_get_assoc () const;
    virtual Go*    	 d_get_parent () const;
    virtual Go*    	 d_get_previous () const;
    virtual Go*    	 d_get_next () const;
    virtual Go*    	 d_get_first_child () const;
    virtual GBool	 d_store_children_array () const;
    virtual void   	 d_set_previous (const Go*);
    virtual void   	 d_set_next (const Go*);
    virtual void   	 d_remove_child (const Go*, int);
    virtual void   	 d_add_child (const Go*, int);
    virtual void         d_set_tree (const Go*, const Go*, const Go*);
    virtual GBool	 d_store_abscs () const;
    virtual GBool	 d_store_relcs () const;
    virtual const GCoordSys& d_get_abscs (const GCoordSys*) const;
    virtual void	 d_set_coordsys (const GCoordSys&, const GCoordSys&,
					 const GCoordSys&);
    virtual GRegion* 	 d_get_clip () const;
    virtual GBool        is_root () const;

    void setup_clips (int = 0);
    void push_clip_postconditions (Go*);
    void call_clip_postconditions ();

    void call_flush_preconditions ();
    void call_flush_postconditions ();
    void expose_handler (GoInterf*, const GExposeEvent*);

    friend void root_expose_handler(void*, GoInterf*, const GExposeEvent*);

    Go* opaque_go_containing_region(const GRegion&, Go*);
    void remove_draw_flags_on_subtree (Go*);
    void redraw_from_unmap (Go*);
    void redraw_from_map (Go*);
    void push_drawings (Go*);
    void push_all_children (Go*);
    void add_to_region (Go*, GRegion&, GBool = GFALSE);
    void add_to_region (const GRegion&, GRegion&);
    GBool subtree_intersect_with_drawing_region (Go*, GBool&);
    void pop_drawings ();
    void set_draw_this (Go*);
    void set_draw_this_recur (Go*);
    Go* upper_go (Go*);
    Go* upper_go_not_child (Go*);


    /*** data ***/
	       
    class GoInterf* _interf;
    GAssoc _assoc;
    Go* _child;
    GCoordSys _cs;
    GRegion _clip;
    GStack _stack;

    GBool has_flush_precond :1;
    GBool has_flush_postcond :1;
    GBool use_drawing_box;
    GBool damage_enabled;
    GNumber min_opaque_square_area;
    int max_boxes_count;
    int max_opaque_search;
    int lower_opaque_search_count;


    GRegion drawing_region;
    GRegion full_drawing_region;
};

#endif /* _GoRoot_ */
