/* $Id: GoStream.h,v 1.4 1992/10/28 16:43:30 mbikina Exp $ */
/*****************************************************************************
MODULE:  GoStream
FILE:    GoStream.h
AUTHOR:  Gilles Benati
DATE:    May 30th, 1991

DESCRIPTION:

	This class has no specific data but defines a general programming
    interface for its subclasses. It provides methods to save and restore
    "anonymous objects" on a Medium (see corresponding man pages). The device
    used depends on the Medium subclass. The following methods may associate a
    Medium with a GoStream instance :

	o  GoStream (GMedium*)
	o  openRead (GMedium*)
	o  openWrite (GMedium*)

    In order for the GoStream to save/restore unknown classes of objects, each
    class that wish to take advantage of it services must implement two
    routines, saveFunc and restoreFunc, and bind them to a string identifying
    its class (ex : metaclass()->classname() for Go's). On the other hand,
    GoStream methods to save and restore objects are saveObj and restoreObj.

    Here is an explanation on how these routines should behave :

	o  int (*saveFunc) (const void* obj , GoStream*);
	   should use the writeXxx (...) methods to save obj data. One may even
	   use saveObj to save other objects. It returns 0 if an error is
	   detected.

	o  int (*restoreFunc) (void** obj, GoStream*);
	   should use the readXxx (...) methods, and even restoreObj method,
	   to read what saveFunc has written, then create a new object instance
	   and return a pointer to it. It returns 0 if an error is detected.

    It is very important that restoreFunc reads items of the same type and 
    in the same order as saveFunc writes them.


COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _GoStream_h
#define _GoStream_h

#include <Go/Tools/Bool.h>
#include <Go/Tools/CoordSys.h>
#include <Go/Tools/Region.h>
#include <Go/Tools/Array.h>
#include <Go/Tools/Assoc.h>
#include <Go/Tools/Rgb.h>


#define  GOPEN_READ              0x01
#define  GOPEN_WRITE             0x02
#define  GFLAG_USER_HEADER       0x04


typedef unsigned long _GId;

class GMedium;

struct _GObjRec {
    void*         obj;
    const char*   cname;
    unsigned long flags;
    unsigned long pos;
    _GObjRec() { pos = flags = 0; obj = NULL; cname = NULL; }
};



class GoStream {

  public:

    static  void	bind(const char* obj_class_name,
			     int (*saveProc) (const void*, GoStream*),
			     int (*restoreProc) (void**, GoStream*));

    GoStream(GMedium* medium = NULL);
    virtual ~GoStream();

    GMedium* medium() { return _medium; };
    virtual int  openRead(GMedium* = NULL);
    virtual int  openWrite(GMedium* = NULL);    
    virtual void close();
    
    virtual int	 rewind();
    virtual int	 forward();
    virtual int  writeHeader(unsigned long , const char*);
    virtual int  readHeader (unsigned long*, char**);
    virtual int  writeFooter();
    virtual int  writeComment(const char*);
    virtual int  deleteFooter();

    virtual int	writeBuffer(unsigned long, const char*, const char* = NULL);
    virtual int	readBuffer(unsigned long*, char**);// **read spec carefully !**;

    virtual int	writeString(const char*, const char* = NULL);
    virtual int	readString(unsigned long, char**);// **read spec carefully !**;

    virtual int	writeInt(int, const char* = NULL);
    virtual int	readInt(int*);

    virtual int	writeBool(GBool, const char* = NULL);
    virtual int	readBool(GBool*);

    virtual int	writeShort(short, const char* = NULL);
    virtual int	readShort(short*);

    virtual int	writeLong(long, const char* = NULL);
    virtual int	readLong(long*);

    virtual int	writeChar(char, const char* = NULL);
    virtual int	readChar(char*);

    virtual int	writeNumber(GNumber, const char* = NULL);
    virtual int	readNumber(GNumber*);

    virtual int	writeDouble(double, const char* = NULL);
    virtual int	readDouble(double*);

    virtual int	writeVector(const GVector&, const char* = NULL);
    virtual int	readVector(GVector*);

    virtual int	writePoint(const GPoint&, const char* = NULL);
    virtual int	readPoint(GPoint*);

    virtual int	writeBox(const GBox&, const char* = NULL);
    virtual int	readBox(GBox*);

    virtual int	writeRgb(const GRgb&, const char* = NULL);
    virtual int	readRgb(GRgb*);

    virtual int	writeCoordSys(const GCoordSys&, const char* = NULL);
    virtual int	readCoordSys(GCoordSys*);

    virtual int	writeRegion(const GRegion&, const char* = NULL);
    virtual int	readRegion(GRegion*);

//	... ?;

    int	saveObj(const void* obj,
		const char* class_name, const char* = NULL);
    int	restoreObj(void** obj, const char (** class_name));

  protected:
    GAssoc		_obj_id_assoc;      // to find id by object's;
    GAssoc		_id_obj_assoc;      // to find object by id's;

    int			_level;
    _GId		_last_id;
    GPointerArray	_to_be_done;        // Contains array of objects to..;
                                            // ..be taken care of (saved);
    GMedium*	        _medium;            // Where  read/write is flushed..;
                                            // ..(may be file, shared memory..);
    unsigned char       _mode  ;            // mode of the stream : read / write

    
    virtual int	 realSave(const void* obj, const void* data, GBool first_obj);

    virtual int	 readObjId(_GId*);
    virtual int	 readObjBegin();
    virtual int	 readObjEnd();
    virtual int  readObj(_GId*);
    virtual int  goToObj(const _GObjRec&);
    virtual int  goToNextObj();
    
    virtual int	 writeObj(_GId, const char* classname, const char* comment);
    virtual int	 writeObjId(_GId);
    virtual int	 writeObjBegin(GBool first_obj);
    virtual int	 writeObjEnd();
    virtual _GId newId(const void*);
    virtual int  readClassName(unsigned long,char **);
    virtual int  saveNullObject();
};



// stores save and restore function by key "classname" in _class_func_assoc;
struct funcRec {
    const char* cname;
    int (*saveFunc) (const void*, GoStream*);
    int (*restoreFunc) (void**, GoStream*);
};


// stores obj id and a state mask (private) by key "object*" in _obj_id_assoc;
struct IdRec {
    _GId          id;
    funcRec     funcs;
    IdRec() {
	id                = (_GId) NULL;
	funcs.cname       = NULL;
	funcs.saveFunc    = NULL;
	funcs.restoreFunc = NULL;
    }
};

#endif /* ndef _GoStream_h */








