/* $Id: Box.h,v 1.2 1992/05/26 16:34:12 benati Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    Box.h
AUTHOR:  Olivier Jojic
DATE:    29 May 1990

DESCRIPTION:
    Box object.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _Box_h
#define _Box_h

#include <Go/machdef.h>
#include <Go/Tools/Point.h>


/*
 * the GBox structure:
 */

struct GBox
{
    GNumber xmin, ymin;
    GNumber xmax, ymax;

    void reset (GNumber xm, GNumber ym, GNumber xM, GNumber yM)
	{ xmin = xm; ymin = ym; xmax = xM; ymax = yM; }

    GBox () {}
    GBox (GNumber xm, GNumber ym, GNumber xM, GNumber yM)
	{ xmin = xm; ymin = ym; xmax = xM; ymax = yM; }
    GBox(const GBox& box)
	{ xmin=box.xmin; ymin=box.ymin; xmax=box.xmax; ymax=box.ymax; }
    
    // build the smallest box containing all points:
    GBox(const GPoint&, const GPoint&);
    GBox(const GPoint&, const GPoint&, const GPoint&, const GPoint&);

    GBox& operator = (const GBox& box) {
	reset(box.xmin, box.ymin, box.xmax, box.ymax);
	return *this;
    }
    
    // displace the box:
    GBox& operator += (const GVector& v) {
	xmin += v.x; xmax += v.x; ymin += v.y; ymax += v.y;
	return *this;
    }
    GBox& operator -= (const GVector& v) {
	xmin -= v.x; xmax -= v.x; ymin -= v.y; ymax -= v.y;
	return *this;
    }

    GNumber width() const { return xmax - xmin; }
    GNumber height() const { return ymax - ymin; }

    inline GBool empty() const { return xmax <= xmin || ymax <= ymin; }

    GBool contains(const GPoint& p) const {
	return p.x > xmin && p.x < xmax && p.y > ymin && p.y < ymax;
    }
    GBool contains(const GPoint& p, GNumber maxdist) const {
	return p.x > xmin - maxdist && p.x < xmax + maxdist &&
	       p.y > ymin - maxdist && p.y < ymax + maxdist;
    }
    
    /*
     * fast methods (not checking empty()) each time:
     */

    friend GBool GEqual(const GBox& a, const GBox& b) {
	return a.xmin == b.xmin && a.xmax == b.xmax &&
	       a.ymin == b.ymin && a.ymax == b.ymax;
    }
    friend GBool GNotEqual(const GBox& a, const GBox& b) {
	return a.xmin != b.xmin || a.xmax != b.xmax ||
	       a.ymin != b.ymin || a.ymax != b.ymax;
    }
    friend GBool GIntersect(const GBox& a, const GBox& b) {
	return a.xmax > b.xmin &&
	       b.xmax > a.xmin &&
	       a.ymax > b.ymin &&
	       b.ymax > a.ymin;
    }

    // return the number of boxes written inside "boxes";
    friend int GUnion(const GBox&, const GBox&, GBox* boxes);

    // return the number of boxes written inside "boxes";
    friend int GSubtraction(const GBox&, const GBox&, GBox* boxes);
    friend GBox GIntersection(const GBox&, const GBox&);

    /*
     * slow methods (checking empty())
     */
    
    friend GBool operator == (const GBox&, const GBox&);
    friend GBool operator != (const GBox&, const GBox&);

    // return GTRUE if the intersection is not empty:
    friend GBool operator && (const GBox&, const GBox&);

    // return the smallest box containing "a union b";
    friend GBox operator + (const GBox& a, const GBox& b);

    // return the smallest box containing a - b:
    friend GBox operator - (const GBox& a, const GBox& b);

    // return the box intersection:
    friend GBox operator & (const GBox&, const GBox&);

    // set this box to the smallest box containing "this union box";
    GBox& operator += (const GBox& box);

    // set this box to the smallest box containing "this minus box";
    GBox& operator -= (const GBox& box);

    // set this to the box intersection:
    GBox& operator &= (const GBox&);
};

const GBox& GEmptyBox();
const GBox& GBiggestBox();

#endif /* _Box_h */
