/* $Id: Colormap.h,v 1.1.1.1 1992/03/18 09:55:16 uep Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    Colormap.h
AUTHOR:  Jacques Davy
DATE:    November 1990

DESCRIPTION:
    This file describes the Colormap object.

    The Colormap object gives a regular interface to the shared object
    ShColormap (see ShColormap.h) and it allows to set some pre/post conditions.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _Colormap_h
#define _Colormap_h

#include <Go/Tools/ShColormap.h>
#include <Go/Tools/Assoc.h>
#include <Go/Tools/Callback.h>

/*
 * Condition flags
 */
struct GColormapCondFlags
{
    GBool _has_modify_precond : 1;
    GBool _has_modify_postcond : 1;
    GBool _has_destroy_postcond : 1;
    GBool _has_addColor_precond : 1;
    GBool _has_addColor_postcond : 1;
    GBool _has_remColor_precond : 1;
    GBool _has_remColor_postcond : 1;
    GBool _has_replaceColor_precond : 1;
    GBool _has_replaceColor_postcond : 1;

    GColormapCondFlags () {
	_has_modify_precond  = GFALSE;
	_has_modify_postcond  = GFALSE;
	_has_destroy_postcond  = GFALSE;
	_has_addColor_precond  = GFALSE;
	_has_addColor_postcond  = GFALSE;
	_has_remColor_precond  = GFALSE;
	_has_remColor_postcond  = GFALSE;
	_has_replaceColor_precond  = GFALSE;
	_has_replaceColor_postcond  = GFALSE;
    }
};

/*
 * GColormap class definition
 */
class GColormap
{
  public:
    GColormap ();
    GColormap (const GColormap* colormap);
    GColormap (const GShColormap* sh_colormap);
    GColormap (ulong mask, const GColormapValues* values, const char* name = NULL);
    GColormap (const char* name);
    ~GColormap ();

    // Getting values
    inline const GShColormap* shColormap() const { return _sh_colormap; }
    inline const GColormapValues* values() const { return _sh_colormap->values(); }

    inline const char* name() const { return _sh_colormap->name(); }
    inline GBool isNamed() const { return _sh_colormap->isNamed(); }

    inline ulong nColor() const { return _sh_colormap->nColor(); }
    inline const GShColor** colors () const { return _sh_colormap->colors(); }

    /* Getting color object, returns NULL if not found */
    inline const GShColor* getColor (const char* name) const { return _sh_colormap->getColor(name); }
    inline const GShColor* getColor (const GRgb* rgb) const { return _sh_colormap->getColor(rgb); }
    inline const GShColor* getColor (uchar gray) const { return _sh_colormap->getColor(gray); }
    inline const GShColor* getColor (const GShPattern* halftone) const { return _sh_colormap->getColor(halftone); }
    inline const GShColor* getColor (ulong index) const;

    /* Getting index, returns -1 if not found */
    inline long getIndex (const char* name) const { return _sh_colormap->getIndex(name); }
    inline long getIndex (const GRgb* rgb) const { return _sh_colormap->getIndex(rgb); }
    inline long getIndex (uchar gray) const { return _sh_colormap->getIndex(gray); }
    inline long getIndex (const GShPattern* halftone) const { return _sh_colormap->getIndex(halftone); }
    inline long getIndex (const GShColor* color) const { return _sh_colormap->getIndex(color); }

    // Modifying methods
    void modify (ulong mask, const GColormapValues* values);
    void modify (const GShColormap*);
    void modify (const char* name);

    long addColor (const GShColor* color); // returns index
    void remColor (const GShColor* color);
    void remColor (ulong index);
    long replaceColor (const GShColor* old_color, const GShColor* new_color);
    long replaceColor (ulong index, const GShColor* new_color);

    // Pre and Post Conditions
    void modifyPrecond (GBool (*)(void*, const GColormap*, ulong&, GColormapValues*),
			const void* closure,
			GAddOrRemove = GADD) const;
    void modifyPostcond (void (*)(void*, GColormap*, ulong), const void* closure,
			 GAddOrRemove = GADD) const;
    void destroyPostcond (void (*)(void*, const GColormap*), const void* closure,
			  GAddOrRemove = GADD) const;
    void addColorPrecond (GBool (*)(void*, const GColormap*, const GShColor*),
			  const void* closure,
			  GAddOrRemove = GADD) const;
    void addColorPostcond (void (*)(void*, GColormap*, ulong),
			   const void* closure,
			   GAddOrRemove = GADD) const;
    void remColorPrecond (GBool (*)(void*, const GColormap*, ulong),
			  const void* closure,
			  GAddOrRemove = GADD) const;
    void remColorPostcond (void (*)(void*, GColormap*, ulong),
			   const void* closure,
			   GAddOrRemove = GADD) const;
    void replaceColorPrecond (GBool (*)(void*, const GColormap*, ulong, const GShColor*),
			      const void* closure,
			      GAddOrRemove = GADD) const;
    void replaceColorPostcond (void (*)(void*, GColormap*, ulong),
			       const void* closure,
			       GAddOrRemove = GADD) const;

    // friends for private callbacks;
    friend void Call_call_modify_postcond   (GColormap* This, const GShColormap*, ulong mask);
    friend void Call_call_addColor_postcond (GColormap* This, const GShColormap*, ulong index);
    friend void Call_call_remColor_postcond (GColormap* This, const GShColormap*, ulong index);
    friend void Call_call_repColor_postcond (GColormap* This, const GShColormap*, ulong index);
				

  private:
    GShColormap* _sh_colormap;

  protected:
    GColormapCondFlags _flags;
    GAssoc _assoc;

    int _addCallback (const char* type, GCBProc proc, const void* closure) const;
    int _remCallback (const char* type, GCBProc proc, const void* closure) const;

    GBool _call_modify_precond (ulong& modify_mask, GColormapValues* values);
    void _call_modify_postcond (ulong modify_mask);
    void _call_destroy_postcond ();
    GBool _call_addColor_precond (const GShColor* color);
    void _call_addColor_postcond (ulong index);
    GBool _call_remColor_precond (ulong index);
    void _call_remColor_postcond (ulong index);
    GBool _call_repColor_precond (ulong index, const GShColor* new_color);
    void _call_repColor_postcond (ulong index);

};

#endif /* _Colormap_h */
