/* $Id: ShColor.h,v 1.4 1992/09/25 11:51:20 smadja Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    ShColor.h
AUTHOR:  Jacques Davy
DATE:    13 June 1990

DESCRIPTION:
    This file describes the Rgb and ShColor objects.
    The Rgb object is a set of three values used to define true colors
    (red, blue, gree).
    The ShColor object is used by the GoInterface when drawing. A ShColor
    object may contain several forms of the same colors:
       * A true color for a color screens
       * A gray value for gray scale screens (0 is black, 0xff is white)
       * An halftone pattern for black and white screens
       * A name
    Depending of the output device characteristics, the GoInterface
    will use the appropriate form of the color (if exist). If the
    appropriate form is not defined, the GoInterface will do as better as
    possible.
    The ShColor object is a shared object. See also Color objects.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#ifndef _ShColor_h
#define _ShColor_h

#include <Go/Tools/SharedObj.h>
#include <Go/Tools/Rgb.h>

/* Mask definitions for GSharedObject operations */
#define GCOLOR_RGB            (1L<<0)
#define GCOLOR_GRAY           (1L<<1)
#define GCOLOR_HALFTONE       (1L<<2)

/* Value structure for GSharedObject operations */
struct GColorValues {
    GRgb rgb;
    uchar gray;
    class GShPattern* halftone;
};


/**
  a GColor tool
  ************
  **/

/** return the brightness of the rgb in the YIQ model **/

inline GRgbValue GBrightness(GRgbValue red, GRgbValue green, GRgbValue blue)
{
    return  (red*30 + green*59 + blue*11) /100;
}

    
/* GShColor class definition */
class GShColor : public GSharedObject
{
    friend class GColor;

  public:

    friend void GModify (GShColor*& sho, ulong mask,
			 const GColorValues* values);
    friend void GModify (const char* name, ulong mask,
			 const GColorValues* values);
    friend void GRename (GShColor*& sho, const char* name);

    static int save(const GShColor*, GoStream*);
    static int restore(GShColor**, GoStream*);
    static const char* classname();
    static int readParams(GShColor*, GoStream*);

    virtual const char* getClassname() const;

    inline GShColor* dup () const;
    inline GBool match (const GShColor* sho) const;
    inline void updatePostcond (void (*)(void*, GShColor*, ulong),
				const void* closure, GAddOrRemove = GADD) const;
    inline void incrRefPostcond (void (*)(void*, GShColor*),
				 const void* closure, GAddOrRemove = GADD) const;
    inline void decrRefPostcond (void (*)(void*, GShColor*),
				 const void* closure, GAddOrRemove = GADD) const;
    inline void destroyPostcond (void (*)(void*, const GShColor*),
				 const void* closure, GAddOrRemove = GADD) const;

    friend GShColor* GCreateShColor ();
    friend GShColor* GCreateShColor (ulong mask, const GColorValues* values,
				     const char* name = NULL);
    friend GShColor* GCreateShColor (const GRgb* rgb, const char* name = NULL);
    friend GShColor* GCreateShColor (uchar gray, const char* name = NULL);
    friend GShColor* GCreateShColor (const GShPattern* halftone,
				     const char* name = NULL);
    friend GShColor* GCreateShColor (const char* name);

    inline const GColorValues* values () const { return &_values; }
    inline const GRgb* rgb () const
    {
	return (_mask & GCOLOR_RGB ? &_values.rgb : NULL);
    }
    inline int gray () const
    {
	return (_mask & GCOLOR_GRAY ? (int)_values.gray : -1);
    }
    inline const class GShPattern* halftone () const
    {
	return (_mask & GCOLOR_HALFTONE ? _values.halftone : NULL);
    }

    inline GRgbValue brightness() const
    {
	GRgbValue ret = 0;
	if (_mask & GCOLOR_GRAY)
	    ret =  _values.gray * 256;
	else if (_mask & GCOLOR_RGB)
	    ret = GBrightness(_values.rgb.red, _values.rgb.green, _values.rgb.blue);
        return ret;
    }
    
    GShPattern* makeHalftone(GBool gray) const;
    
  protected:

    inline GList*& _head () { return _headOfList; }
    inline int _valueSize () { return (sizeof(GColorValues)); }
    virtual GBool _match (ulong mask, const void* values) const;
    virtual void _makeNewValues (ulong mask, const void* values,
				 ulong& newMask, void* newValues) const;
    virtual ulong _update (ulong mask, const void* values);
    friend GShColor* _newGShColor (GFunctionList* funcList, ulong mask,
				   const void* values,
				   const char* name = NULL);

    GShColor (GFunctionList* funcList, ulong mask, const GColorValues* values,
	      const char* name = NULL);
    virtual ~GShColor ();


  private:

    GColorValues _values;
    static GList* _headOfList;
    static GFunctionList _funcList;
};

/*
 * Inline Methods
 */
inline void GModify (GShColor*& sho, ulong mask, const GColorValues* values)
{
    sho->modify((GSharedObject*&)sho, mask, (const void*)values);
}

/**/
inline void GRename (GShColor*& sho, const char* name)
{
   sho->rename((GSharedObject*&)sho, name);
}

/**/
inline GShColor* GShColor::dup () const
{
    return (GShColor*)((GSharedObject*)this)->dup();
}

/**/
inline GBool GShColor::match (const GShColor* sho) const
{
    return ((GShColor*)this)->_match (sho->mask(), sho->values());
}

/**/
inline void GShColor::updatePostcond (void f(void*, GShColor*, ulong),
				     const void* closure, GAddOrRemove add) const
{
    GSharedObject::updatePostcond (
   (void(*)(void*, GSharedObject*, ulong))f, closure, add);
}

/**/
inline void GShColor::incrRefPostcond (void f(void*, GShColor*),
				      const void* closure, GAddOrRemove add) const
{
    GSharedObject::incrRefPostcond (
		          (void(*)(void*, GSharedObject*))f, closure, add);
}

/**/
inline void GShColor::decrRefPostcond (void f(void*, GShColor*),
				      const void* closure, GAddOrRemove add) const
{
    GSharedObject::decrRefPostcond (
                         (void(*)(void*, GSharedObject*))f, closure, add);
}

/**/
inline void GShColor::destroyPostcond (void f(void*, const GShColor*),
				      const void* closure, GAddOrRemove add) const
{
    GSharedObject::destroyPostcond (
		          (void(*)(void*, const GSharedObject*))f, closure, add);
}

#endif /* _ShColor_h */
