/* $Id: ShPixelArray.h,v 1.4 1992/09/25 11:52:29 smadja Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    ShPixelArray.h
AUTHOR:  Jacques Davy
DATE:    14 June 1990

DESCRIPTION:
    The ShPixelArray object is defined by a array of pixels (its data) and
    some parameters: width, height, depth, bit_per_pixels, padding, bitmap
    unit, byte and bit orders. The format used in the ShPixelArray value is
    the Z-Format (a set of pixel values in scanline order).
        ========

    The ShPixelArray object is a shared object and is used by Pattern
    and GoImage objects. As most of defined SharedObjects, the ShPixelArray
    may have a name.

    The ShPixelArray does not provide ediding methods, but it is possible
    to extract a "subPixelArray" or to create a new scaled ShPixelArray.

COPYRIGHT :    
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  license from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _ShPixelArray_h
#define _ShPixelArray_h

#include <Go/machdef.h>
#include <Go/Tools/Bool.h>
#include <Go/Tools/SharedObj.h>
#include <Go/Tools/GCdef.h>
#include <Go/Tools/Array.h>
#include <Go/Tools/CoordSys.h>
#include <Go/Tools/Number.h>

/* Flag definitions */
#define GLSB_FIRST 0
#define GMSB_FIRST 1
#define GBITMAP_CODING   0
#define GGROUP3          1
#define GGROUP4          2

/* Mask definitions for GSharedObject operations */

#define GPXL_VALUE         (1L<<0)
#define GPXL_SHARED        (1L<<1)
#define GPXL_GIVEN         (1L<<2)

enum GDataSpace
{
    GDATA_COPY = 0,
    GDATA_SHARE = GPXL_SHARED,
    GDATA_GIFT = GPXL_GIVEN
} ;


/* Value structure for GSharedObject operations */
struct GPixelArrayValues
{
    ushort    width;           // number of bitmapUnits in a line (scanline)
    ushort    height;          // number of lines in a plane
    ushort    xoffset;         // number of pixels offset in X direction
    uchar     bitmap_unit;     // number of bits in a scanline Unit (8, 16, 32)
    uchar     padding;         // scanline padding (8, 16, 32)
    uchar     bits_per_pixel;  // pixel value coding size
    uchar     depth;           // number of bits per pixels
    struct
    {
        unsigned bit  : 1; // bit order (LSBFIRST, MSBFIRST) in the bitmap unit
        unsigned byte : 1; // byte order (  "    ,    "    ) in the pixel value
	unsigned coding : 2;  // Encoding format (BITMAP_CODING, GROUP3 or 4) 
    } format;
    uchar*     data;          // pixel array
};

/* GShPixelArray class definition */
class GShPixelArray : public GSharedObject
{
    friend class GPixelArray;

  public:
    
    static int save(const GShPixelArray*, GoStream*);
    static int restore(GShPixelArray**, GoStream*);
    static const char* classname();
    static int readParams(GShPixelArray*, GoStream*);

    virtual const char* getClassname() const;
    
    friend void   	GModify  (GShPixelArray*& sho,
				  ulong mask, const GPixelArrayValues* values);
    friend void         GModify  (const char* name,
				  ulong mask, const GPixelArrayValues* values);
    friend void         GRename  (GShPixelArray*& sho, const char* name);

    inline GShPixelArray* dup() const;
    inline GBool          match (const GShPixelArray* sho) const;

    inline void updatePostcond  (void (*)(void*, GShPixelArray*, ulong),
				 const void* closure, GAddOrRemove = GADD) const;
    inline void incrRefPostcond (void (*)(void*, GShPixelArray*),
				 const void* closure, GAddOrRemove = GADD) const;
    inline void decrRefPostcond (void (*)(void*, GShPixelArray*),
				 const void* closure, GAddOrRemove = GADD) const;
    inline void destroyPostcond (void (*)(void*, const GShPixelArray*),
				 const void* closure, GAddOrRemove = GADD) const;
    
    friend GShPixelArray* GCreateShPixelArray (ulong mask,
					       const GPixelArrayValues* values,
					       const char* name = NULL,
					       GDataSpace = GDATA_COPY);
    friend GShPixelArray* GCreateShPixelArray (const char* name);
    friend GShPixelArray* GCreateShPixelArray ();
    
    inline const  GPixelArrayValues* values () const { return &_values; }
    inline ulong  bytesPerLine () const { return _bytes_per_line; }
#if defined(__BORLANDC__) || defined(__MSC7__)
    inline ulong  dataSize () const { return _bytes_per_line * _values.height;}
#else
    inline ulong  dataSize () const { return (((ulong)_bytes_per_line) &0x0ffffL) * _values.height;}
#endif
    GShPixelArray* subPixelArray (ushort x, ushort y,
				  ushort width, ushort height) const;
    GShPixelArray* subPixelArray (GNumber x, GNumber y,
				  GNumber width, GNumber height) const;
    void          deformation   (GPixelArrayValues* result,
				 ushort* xorigin_result,
				 ushort* yorigin_result,
				 const GCoordSys& cs,
				 const GArray* pixel_translation,
				 uchar bits_per_pixel_of_result,
				 uchar depth_of_result,
				 uchar bit_order_of_result,
				 uchar byte_order_of_result,
				 GQuality quality) const;
    
    void   subPixelArrayValues  (ushort x, ushort y,
				 ushort width, 
				 ushort height,
				 GPixelArrayValues&) const;
    
  protected:

    inline GList*&      _head () { return _headOfList; }
    inline int          _valueSize () { return (sizeof(GPixelArrayValues)); }
    virtual GBool       _match (ulong mask, const void* values) const;
    virtual void        _makeNewValues (ulong mask, const void* values,
					ulong& newMask, void* newValues) const;
    virtual ulong       _update (ulong mask, const void* values);
    friend GShPixelArray* _newGShPixelArray (GFunctionList* funcList, ulong mask,
					     const void* values,
					     const char* name = NULL);

    GShPixelArray (GFunctionList* funcList, ulong mask,
		   const GPixelArrayValues* values,
		   const char* name = NULL);
    virtual ~GShPixelArray ();


    
  private:

    GPixelArrayValues    _values;
    static GList*        _headOfList;
    static GFunctionList _funcList;
    ushort              _bytes_per_line;

};

/*
 * Inline Methods
 */
inline void GModify (GShPixelArray*& sho, ulong mask,
			   const GPixelArrayValues* values)
{
    sho->modify((GSharedObject*&)sho, mask, (const void*)values);
}

/**/
inline void GRename (GShPixelArray*& sho, const char* name)
{
   sho->rename((GSharedObject*&)sho, name);
}

/**/
inline GShPixelArray* GShPixelArray::dup() const
{
    return (GShPixelArray*)((GSharedObject*)this)->dup();
}

/**/
inline GBool GShPixelArray::match (const GShPixelArray* sho) const
{
    return ((GShPixelArray*)this)->_match (sho->mask(), sho->values());
}

/**/
inline void GShPixelArray::updatePostcond (void cb(void*, GShPixelArray*, ulong),
					  const void* closure, GAddOrRemove action) const
{
    GSharedObject::updatePostcond ((void(*)(void*, GSharedObject*, ulong))cb, closure,
				  action);
}

/**/
inline void GShPixelArray::incrRefPostcond (void cb(void*, GShPixelArray*),
					   const void* closure, GAddOrRemove action) const
{
    GSharedObject::incrRefPostcond ((void(*)(void*, GSharedObject*))cb, closure,
				   action);
}

/**/
inline void GShPixelArray::decrRefPostcond (void cb(void*, GShPixelArray*),
					   const void* closure, GAddOrRemove action) const
{
    GSharedObject::decrRefPostcond ((void(*)(void*, GSharedObject*))cb, closure,
				   action);
}

/**/
inline void GShPixelArray::destroyPostcond (void cb(void*, const GShPixelArray*),
					   const void* closure, GAddOrRemove action) const
{
    GSharedObject::destroyPostcond ((void(*)(void*, const GSharedObject*))cb, closure,
				   action);
}

#endif /* _ShPixelArray_h */
