/* $Id: Undo.h,v 1.3 1992/12/11 14:18:43 smadja Exp $ */
/*****************************************************************************
         Copyright (c) BULL S.A. 1990

MODULE:  Go Tools
FILE:    Undo.h
AUTHOR:  Oliver Jojic
DATE:    Nov 25, 1991

DESCRIPTION:
    This class try to help you in undo storage and management.
    Main methods that this class offers are:
    	- pushUndo
	- undo
	- redo

    First we will define some words:
    	- an action (Ai) is a functionality which has a corresponding
	  undo action.
	- an undo action (UAi) is the undo action of Ai.
	  The undo action of UAi is often (but not necessarely) Ai.

    A simple implementation of undo management will use a stack to
    push every undo actions.

    Therefore after the execution of n actions Ai we will have this stack:
    	UA1 ... UAn

    And a call of `undo' will execute UAn and pop the last element of
    the stack:
    	UA1 ... UAn-1

    The problem of this implementation is that when you execute many
    undos you cannot go back and undo `undo actions'.

    So, a better solution is to store a list (and not a stack) with a
    pointer to the current undo action.
    In the following we will consider that Ai is the undo action of UAi
    (and not only that UAi is the undo action of Ai). This is not mandatory
    but makes all the following more readable.

    After n actions, we get:
    	UA1 ... UAn
	           ^	(^ is the pointer)
	       
    after 1 undo, we get:
    	UA1 ... UAn-1 An
		     ^

    and after some undos, we get:
    	UA1 ... UAi Ai+1 ... An
    		   ^

    now, if we call `redo', we get:
    	UA1 ... UAi UAi+1 Ai+2 ... An
		         ^

    This way, we can navigate (i.e. undo and redo) from (U)A1 to (U)An.

    We must also explain what is done when someone execute an action B while
    the undo stack is in this state: 	UA1 ... UAi Ai+1 ... An
    						   ^
    The UndoManager class removes all undo actions after the pointer and
    stores UB:
    	UA1 ... UAi UB
		      ^

    All this (somewhat complicated) undo management is managed by the
    UndoManager class. The only method that programmers have to call inside
    actions is `push()'. They do not have to care about undo undo !

    As an example, look at these two complementary methods:

    	void select (someObject) {
	    ... select part ...
	    undoMgr.pushUndo(unselect, someObject);
	}

	void unselect (someObject) {
	    ... unselect part ...
	    undoMgr.pushUndo(select, someObject);
	}
	
    It is the job of UndoManager to know if the pushUndo'ed action is to
    be inserted as an undo action or as an undo-undo action, that is left
    of the pointer or right of the pointer.
    

Some more features:

  * You can set the maximum count of stored undo actions. If this maximum
    is reached, new undo actions are always stored but first undo actions are
    removed.
    The default maximum count is `defaultUndoActionsMaxCount'.
    `maxCount()' returns the maximum count;
    `maxCount(int)' sets this maximum count.

  * At any time you can start an undo mark, do many actions and end the undo
    mark. When the method undo() (or redo()) find such a mark, it
    continues the undo until the end of the mark.
    Undo marks may be nested. In this case the outermost is used.
    
  * The method reset(), makes the undo list empty.

  * Often, an action need to store many informations to be able to realize
    the correct undo. This information will be allocated via malloc() (or
    any memory allocator). Therefore when the undo is performed, or if the
    UndoManager instance is reseted, someone must free this memory.
    The third parameter of push() is the method which must free this
    memory:
      - push(undoProc, data)
      		do not have any free procedure.
      - push(undoProc, data, freeProc)
      		will call freeProc with data as parameter:
			freeProc(data)
      - push(undoProc, data, freeProc, freeData)
      		will call freeProc with freeData as parameter:
			freeProc(freeData)

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _Undo_h
#define _Undo_h

#include <Go/Tools/Queue.h>

typedef void (*GUndoProc)(void*);

extern const int gDefaultUndoActionsMaxCount;

class GUndoManager
{
  public:
    GUndoManager();
   ~GUndoManager();

    void push (GUndoProc undoProc, void* closure,
	       GUndoProc freeProc = NULL, void* freeClosure = NULL);

    void undo ();
    void redo ();
    void undoUndo () { redo (); }
	 
    GBool undoEmpty () const;
    GBool redoEmpty () const;
    GBool undoUndoEmpty () const { return redoEmpty (); }
	

    int maxCount () const;
    void maxCount (int);

    void openMark ();
    void closeMark ();

    void reset ();
    void removeOneUndo ();
    void removeOneRedo ();

  private:
    void reset_redo_queue();
    void reset_undo_queue();
    void call_undo();
    void incr_count();
    void get_first_undo(int&, void(*&)(void*), void*&);
    void get_last_undo(GQueue*, int&,
		       void(*&)(void*), void*&,
		       void(*&)(void*), void*&);
    void add_last_undo(void(*)(void*), void*, void(*)(void*), void*);
    void remove_first_undo();

    GQueue undo_queue;
    GQueue redo_queue;

    enum { outside, inside_undo, inside_redo } _state;

    int   unclosed_mark_count;	//{ unclosed_mark_count >= 0 }

    int   max_count;		//{ 1 <= max_count }
    int   _count;		//{ 0 <= _count <= max_count }
};

#endif /* _Undo_h */
