/* $Id: Vector.h,v 1.1.1.1 1992/03/18 09:58:53 uep Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    Vector.h
AUTHOR:  Olivier Jojic
DATE:    Jan 90

DESCRIPTION:

The Vector class with classic functions.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _Vector_h
#define _Vector_h

#include <Go/machdef.h>
#include <Go/Tools/Number.h>
#include <Go/Tools/Bool.h>

struct GVector
{
    GNumber x, y;

    GVector();
    GVector(GNumber, GNumber);
    GVector(const GVector&);

    GNumber squareSize() const;
    GNumber size() const;
    GNumber angle() const;
    GBool   null() const;

    GVector operator - () const;
    GVector& operator += (const GVector&);
    GVector& operator -= (const GVector&);
    GVector& operator *= (GNumber);
    GVector& operator /= (GNumber);
    GVector& neg();
    GVector& resize(GNumber);
    GVector& rotate(GNumber);
    GVector& setAngle(GNumber);

    friend GBool operator == (const GVector&, const GVector&);
    friend GBool operator != (const GVector&, const GVector&);

    friend GVector operator + (const GVector&, const GVector&);
    friend GVector operator - (const GVector&, const GVector&);
    friend GVector operator * (const GVector&, GNumber);
    friend GVector operator * (GNumber, const GVector&);
    friend GVector operator / (const GVector&, GNumber);

    friend GNumber operator * (const GVector&, const GVector&);
    friend GVector& operator *= (GVector&, const class GMatrix&);

    friend GNumber GAngle (const GVector&, const GVector&);
};


struct GUnitVector : public GVector
{
    GUnitVector(GNumber angle);
    GUnitVector(const GVector&);
};

/*
 * inline methods:
 */


inline GVector::GVector ()
{
    x = y = 0;
}

inline GVector::GVector (GNumber a, GNumber b)
{
    x = a;
    y = b;
}

inline GVector::GVector (const GVector& v)
{
    x = v.x;
    y = v.y;
}

inline GNumber GVector::squareSize () const
{
    return x * x + y * y;
}

inline GNumber GVector::size () const
{
    return sqrt(squareSize());
}

inline GBool GVector::null () const
{
    return x == 0 && y == 0;
}

inline GVector GVector::operator - () const
{
    return GVector(-x, -y);
}

inline GVector& GVector::operator += (const GVector& v)
{
    x += v.x;
    y += v.y;
    return *this;
}

inline GVector& GVector::operator -= (const GVector& v)
{
    x -= v.x;
    y -= v.y;
    return *this;
}

inline GVector& GVector::operator *= (GNumber n)
{
    x *= n;
    y *= n;
    return *this;
}

inline GVector& GVector::operator /= (GNumber n)
{
    x /= n;
    y /= n;
    return *this;
}

inline GVector& GVector::neg ()
{
    x = -x;
    y = -y;
    return *this;
}

inline GVector& GVector::resize (GNumber nsz)
{
    return *this *= (nsz / size());
}

inline GVector& GVector::setAngle (GNumber angle)
{
    GNumber sz = size();
    x = sz * cos(angle);
    y = sz * sin(angle);
    return *this;
}

inline GVector& GVector::rotate (GNumber incr)
{
    return setAngle(angle() + incr);
}

inline GBool operator == (const GVector& a, const GVector& b)
{
    return a.x == b.x && a.y == b.y;
}

inline GBool operator != (const GVector& a, const GVector& b)
{
    return !(a == b);
}

inline GVector operator + (const GVector& a, const GVector& b)
{
    return GVector(a.x + b.x, a.y + b.y);
}

inline GVector operator - (const GVector& a, const GVector& b)
{
    return GVector(a.x - b.x, a.y - b.y);
}

inline GVector operator * (const GVector& v, GNumber n)
{
    return GVector(v) *= n;
}

inline GVector operator * (GNumber n, const GVector& v)
{
    return v * n;
}

inline GVector operator / (const GVector& v, GNumber n)
{
    return GVector(v) /= n;
}

inline GNumber operator * (const GVector& a, const GVector& b)
{
    return a.x * b.x + a.y * b.y;
}


inline GUnitVector::GUnitVector (GNumber angl) : GVector(cos(angl), sin(angl))
{
}

inline GUnitVector::GUnitVector (const GVector& v) : GVector(v)
{
    resize(1);
}

#endif /* _Vector_h */
