/* $Id: mGo.h,v 1.3 1992/10/19 12:25:01 jojic Exp $ */
/*****************************************************************************
MODULE:  Go Kernel
FILE:    mGo.h
AUTHOR:  Oliver Jojic
DATE:    April 91

DESCRIPTION:
  This file describe the GoMetaclass.

  The GoMetaclass allows you:
  	- to get class informations (name, superclass, ...),
	- to make actions at class level
	  (or on every instance of the corresponding go class).

  Each "class GoSub" has a corresponding "class GoSubMetaclass".
  (if GoSub is a subclass of GoSup, GoSubMetaclass is a subclass
   of GoSupMetaclass).

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _GoMetaclass_
#define _GoMetaclass_

#include <Go/machdef.h>
#include <Go/Go.h>
#include <Go/Tools/Assoc.h>
#include <Go/Tools/Set.h>
#include <Go/Tools/Callback.h>

class GoMetaclass;
class GoRoot;
class GoInterf;
class GoStream;


/*
 * An ActionRec is used to give the proc associated to a name.
 * Each GoMetaclass has its ActionRec list
 * (terminated by a {NULL, NULL}).
 */
typedef void (*GActionProc)();
struct GActionRec
{
    const char*  name;
    GActionProc  proc;
};


class GoSet : public GSet
{
  public:
    Go*  first () const { return (Go*)GSet::first(); }
    Go*  next () const { return (Go*)GSet::next(); }
};


class GoMSet : public GSet
{
  public:
    GoMetaclass* first () const { return (GoMetaclass*)GSet::first(); }
    GoMetaclass* next () const { return (GoMetaclass*)GSet::next(); }
};



/***********************************************
*
*		  Go metaclass
*
***********************************************/

class GoMetaclass
{
  public:
    
    /*** class informations ***/

    const char*      	   classname () const;
    GoMetaclass*     	   superclass () const;
    const GoMSet&          subclasses () const;
    GBool                   isSubclass (const GoMetaclass*) const;

    static const GoMSet&   metaclasses ();


    /*** instance informations ***/

    const GoSet&	   instances () const;
    const GActionRec* 	   actions () const;


    /*** misc. ***/
    
    GAssoc&           	   assoc () const;
    

    /*** pre and post conditions ***/

    // go creation and go destruction conditions:

    void  	createPrecond	(GBool(*)(void*, const GoMetaclass*),
				 void* closure, GAddOrRemove=GADD);
    void  	createPostcond (void(*)(void*, Go*),
				void* closure, GAddOrRemove=GADD);
    void  	dupPrecond (GBool(*)(void*, const Go*),
			    void* closure, GAddOrRemove=GADD);
    void  	dupPostcond (void(*)(void*, Go* source, Go* copy),
			     void* closure, GAddOrRemove=GADD);
    void  	destroyPrecond (GBool(*)(void*, const Go*),
				void* closure, GAddOrRemove=GADD);
    void  	destroyPostcond (void(*)(void*, const Go*),
				 void* closure, GAddOrRemove=GADD);

    // tree structure conditions:

    void  	shiftPrecond (GBool(*)(void*, const Go*, const Go*(*pnextbr)),
			      void* closure, GAddOrRemove=GADD);
    void  	shiftPostcond (void(*)(void*, Go*, Go* oldnextbr),
			       void* closure, GAddOrRemove=GADD);
    void	removeChildPrecond (GBool(*)(void*,const Go*,const Go*chtorem),
				    void* closure, GAddOrRemove=GADD);
    void  	removeChildPostcond (void(*)(void*,Go*,Go* remch, Go* onext),
				     void* closure, GAddOrRemove=GADD);
    void  	addChildPrecond (GBool(*)(void*, const Go*,
					 const Go* addch, const Go*(*pnext)),
				 void* closure, GAddOrRemove=GADD);
    void  	addChildPostcond (void(*)(void*, Go*, Go* addch),
				  void* closure, GAddOrRemove=GADD);
    void  	reparentPrecond (GBool(*)(void*, const Go*,
					 const Go* newpar, const Go*(*pnext)),
				 void* closure, GAddOrRemove=GADD);
    void  	reparentPostcond (void(*)(void*, Go*, Go* oldp, Go* oldnext),
				  void* closure, GAddOrRemove=GADD);
    void  	rootPrecond (GBool(*)(void*, const Go*, const Go* newroot),
			     void* closure, GAddOrRemove=GADD);
    void  	rootPostcond (void(*)(void*, Go*, Go* oldroot),
			      void* closure, GAddOrRemove=GADD);
    void  	crossTypePrecond (GBool(*)(void*, const Go*, GCrossType*),
				  void* closure, GAddOrRemove=GADD);
    void  	crossTypePostcond (void(*)(void*, Go*, const GCrossType* old),
				   void* closure, GAddOrRemove=GADD);

    // coordinate system conditions:

    void  	relcsPrecond (GBool(*)(void*, const Go*, GCoordSys* newrel,
				      const GCoordSys* oldparabs,
				      const GCoordSys* newparabs),
			      void* closure, GAddOrRemove=GADD);
    void  	relcsPostcond (void(*)(void*, Go*, const GCoordSys* parabs),
			       void* closure, GAddOrRemove=GADD);
    void  	abscsPrecond (GBool(*)(void*, const Go*, GCoordSys* newabs,
				      const GCoordSys* oldparabs,
				      const GCoordSys* newparabs),
			      void* closure, GAddOrRemove=GADD);
    void  	abscsPostcond (void(*)(void*, Go*, const GCoordSys* parabs),
			       void* closure, GAddOrRemove=GADD);

    // drawing conditions:

    void  	mapPrecond (GBool(*)(void*, const Go*),
			    void* closure, GAddOrRemove=GADD);
    void  	mapPostcond (void(*)(void*, Go*),
			     void* closure, GAddOrRemove=GADD);
    void  	selectPrecond (GBool(*)(void*, const Go*),
			       void* closure, GAddOrRemove=GADD);
    void  	selectPostcond (void(*)(void*, Go*),
				void* closure, GAddOrRemove=GADD);
    void	 myClipPostcond (void(*)(void*, Go*),
				 void* closure, GAddOrRemove=GADD);
    void	 clipPostcond (void(*)(void*, Go*),
			       void* closure, GAddOrRemove=GADD);

    // misc. conditions:
    
    void  	interfPrecond (GBool(*)(void*, const Go*, const GoInterf*),
			       void*, GAddOrRemove=GADD);
    void  	interfPostcond (void(*)(void*, Go*, GoInterf* oldinterf),
				void* closure, GAddOrRemove=GADD);
    void  	savePrecond (GBool(*)(void*, const Go*),
			     void* closure, GAddOrRemove=GADD);
    void  	savePostcond (void(*)(void*, const Go*, GoStream*),
			      void* closure, GAddOrRemove=GADD);
    void 	restorePostcond (void(*)(void*, Go*, GoStream*),
				 void* closure, GAddOrRemove=GADD);

    GBool	CallPreconditions (const char*, const Go*) const;
    GBool	CallPreconditions (const char*, const Go*, void*) const;
    GBool	CallPreconditions (const char*, const Go*, void*, void*) const;
    GBool	CallPreconditions (const char*, const Go*,
				   void*, void*, void*) const;
    GBool	CallPreconditions (const char*, const Go*,
				   void*, void*, void*, void*) const;

    void        CallPostconditions (const char*, Go*);
    void        CallPostconditions (const char*, Go*, void*);
    void        CallPostconditions (const char*, Go*, void*, void*);
    void        CallPostconditions (const char*, Go*, void*, void*, void*);
    void        CallPostconditions (const char*, Go*,
				    void*, void*, void*, void*);

  protected:
    		GoMetaclass (); // to get the GoMetaclass use:
    virtual    ~GoMetaclass (); //	Go::metaclass()

    GBool	AddOrRemoveCondition(const char*, void* proc, void* closure,
				     GAddOrRemove);
  public: //should be protected
    void	AddSubclass (const GoMetaclass*);
    void	RemoveSubclass (const GoMetaclass*);


  private:

    friend Go;
    friend GoRoot;

    /*** virtual methods ***/

    virtual const char* 	m_classname () const;
    virtual GoMetaclass*	m_superclass () const;
    virtual const GActionRec*	m_actions () const;
    virtual Go*			m_create () const;
    
    /*** private data ***/

    GoMSet 	   sub_classes;
    static GoMSet* meta_classes;
    GoSet  	   _instances;
    GAssoc  	   _assoc;

    GBool has_create_precond :1;
    GBool has_dup_precond :1;
    GBool has_destroy_precond :1;
    GBool has_save_precond :1;
    GBool has_shift_precond :1;
    GBool has_reparent_precond :1;
    GBool has_remove_child_precond :1;
    GBool has_add_child_precond :1;
    GBool has_root_precond :1;
    GBool has_interf_precond :1;
    GBool has_cross_type_precond :1;
    GBool has_relcs_precond :1;
    GBool has_abscs_precond :1;
    GBool has_map_precond :1;
    GBool has_select_precond :1;

    GBool has_create_postcond :1;
    GBool has_destroy_postcond :1;
    GBool has_save_postcond :1;
    GBool has_restore_postcond :1;
    GBool has_dup_postcond :1;
    GBool has_shift_postcond :1;
    GBool has_reparent_postcond :1;
    GBool has_remove_child_postcond :1;
    GBool has_add_child_postcond :1;
    GBool has_root_postcond :1;
    GBool has_interf_postcond :1;
    GBool has_cross_type_postcond :1;
    GBool has_relcs_postcond :1;
    GBool has_abscs_postcond :1;
    GBool has_map_postcond :1;
    GBool has_select_postcond :1;
    GBool has_my_clip_postcond :1;
    GBool has_clip_postcond :1;
};



/**********************************************
*
*		inline methods
*
***********************************************/

INLINE const char* GoMetaclass::classname () const
{
    return m_classname();
}

INLINE GoMetaclass* GoMetaclass::superclass () const
{
    return m_superclass();
}

INLINE const GoMSet& GoMetaclass::subclasses () const
{
    return sub_classes;
}

INLINE const GoMSet& GoMetaclass::metaclasses ()
{
    return *(GoMetaclass::meta_classes);
}

INLINE const GoSet& GoMetaclass::instances () const
{
    return _instances;
}

INLINE const GActionRec* GoMetaclass::actions () const
{
    return m_actions();
}

INLINE GAssoc& GoMetaclass::assoc () const
{
    const GAssoc* assoc = &_assoc;
    return (GAssoc&)*assoc;
}

INLINE void GoMetaclass::AddSubclass (const GoMetaclass* mc)
{
    sub_classes.add(mc);
}

INLINE void GoMetaclass::RemoveSubclass (const GoMetaclass* mc)
{
    sub_classes.remove(mc);
}

#endif /* _GoMetaclass_ */
